<?php
// +----------------------------------------------------------------------
// Swoole - HTTP-SERVER-API
// +----------------------------------------------------------------------
// Copyright (c) 2018 https://xiuxian.junphp.com All rights reserved.
// +----------------------------------------------------------------------
// Licensed ( http://www.apache.org/licenses/LICENSE-2.0 )
// +----------------------------------------------------------------------
// Author: 小黄牛 <1731223728@qq.com>
// +----------------------------------------------------------------------
ini_set('display_errors', 1); 
error_reporting(E_ALL ^ E_NOTICE);
require_once dirname(__FILE__).'/vendor/api.php';

$server = new server();
$server->run();

class server{
	/**
	 * WS的启动实例
	*/
	private $_ws;
	/**
	 * host-IP，0.0.0.0表示允许接收所有请求
	*/
	private $_host = '0.0.0.0';
	/**
	 * 端口号
	*/
	private $_port = '9501';

    /**
     * 这是启动服务端的入口
    */
    public function run() { 
		$this->start();
		$this->onRequest();
		$this->_ws->start();
    }

	/**
	 * 启动http-server服务
	*/
	private function start() {
		# 创建websocket服务器对象，监听0.0.0.0:9502端口
		$this->_ws = new swoole_http_server($this->_host, $this->_port);
		$this->_ws->set([
			'worker_num' => 4,
			'max_request' => 500
		]);
	}

	/**
	 * 监听回调请求
	*/
	private function onRequest() {
		$this->_ws->on('request', function ($request, $response) {
			# 防止Chrome的空包
			$uri = $request->server['request_uri'];
			
			if ($uri == '/favicon.ico') {
				$response->status(404);
				$response->end();
			} else {
				# 先验证下请求头
				api::check($request->header);
				# 先验证下SIGN
				$data = api::param($request);
				$res = api::sign_vif($data);
				if ($res !== true) {
					$response->end($res);
				} else {
					# 继续向下执行
					$this->follow_up_route($request, $response, $data);
				}
			}
		});
	}

	/**
	 * 请求API检测
	*/
	private function follow_up_route($request, $response, $param) {
		$referer = ltrim($request->server['request_uri'], '/');
		# 分割路由
		$array = explode('/', $referer);

		if (count($array) < 2) {
			$response->end(api::json('50001', '请求地址错误，格式为：版本号/业务名/接口名'));
		} else {
			$path = dirname(__FILE__).'/app';
			$edition = $path.'/'.$array[0];
			if (!file_exists($edition)) {
				$response->end(api::json('50002', '暂无该版本'));
			} else {
				$controller = $edition.'/'.$array[1];
				if (!file_exists($controller)) {
					$response->end(api::json('50003', '暂无该业务'));
				} else {
					if (!empty($array[2])) {
						$action_name = $array[2];
					} else {
						$action_name = 'index';
					}
					$action = $controller.'/'.$action_name.'.php';
					if (!file_exists($action)) {
						$response->end(api::json('50004', '暂无该接口'));
					} else {
						$this->follow_up_transmit($request, $response, $param, $action, $action_name);
					}
				}
			}
		}
	}

	/**
	 * 请求接口转发
	*/
	private function follow_up_transmit($request, $response, $param, $action, $action_name) {
		# 引入接口文件
		require_once $action;
		# 实例化接口
		$obj = new $action_name();
		$obj->run($request, $response, $param, new api);
	}
}
